package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.WrongValueException;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Messagebox;

import com.rjconsultores.ventaboletos.entidad.Categoria;
import com.rjconsultores.ventaboletos.entidad.Pacote;
import com.rjconsultores.ventaboletos.entidad.PacoteTarifa;
import com.rjconsultores.ventaboletos.entidad.TipoTarifaPacote;
import com.rjconsultores.ventaboletos.service.CategoriaService;
import com.rjconsultores.ventaboletos.service.PacoteTarifaService;
import com.rjconsultores.ventaboletos.utilerias.BigDecimalUtil;
import com.rjconsultores.ventaboletos.utilerias.LocaleUtil;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextboxDecimal;

@Controller("editarPacoteTarifaController")
@Scope("prototype")
public class EditarPacoteTarifaController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	private static Logger log = LogManager.getLogger(EditarPacoteTarifaController.class);

	@Autowired
	private PacoteTarifaService pacoteTarifaService;
	
	@Autowired
	private CategoriaService categoriaService;
	
	private List<Categoria> lsCategorias;
	private Combobox cmbCategoria;

	private Pacote pacote;
	private TipoTarifaPacote tipoTarifaPacote;
	private MyListbox tarifaPacoteList;
	private MyTextbox txtDescTarifa;
	private MyTextboxDecimal tarifa;
	
	private Checkbox chkIndvendaweb;
	
	PacoteTarifa pacoteTarifa;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		lsCategorias = categoriaService.obtenerTodos();
		pacoteTarifa = (PacoteTarifa) Executions.getCurrent().getArg().get("pacoteTarifa");
		pacote = (Pacote) Executions.getCurrent().getArg().get("pacote");
		tipoTarifaPacote = (TipoTarifaPacote) Executions.getCurrent().getArg().get("tipoTarifaPacote");
		tarifaPacoteList = (MyListbox) Executions.getCurrent().getArg().get("tarifaPacoteList");

		if (tipoTarifaPacote != null) {
			txtDescTarifa.setValue(tipoTarifaPacote.getDesctipotarifa());
		}
		
		if(pacoteTarifa != null) {
			tarifa.setValue(BigDecimalUtil.getBigDecimalToStringDouble2CasasDecimaisFormatado(pacoteTarifa.getTarifa(), LocaleUtil.getLocale()));
			chkIndvendaweb.setChecked(pacoteTarifa.getIndvendaweb() != null ? pacoteTarifa.getIndvendaweb() : false);
			
			if(pacoteTarifa.getCategoria() != null) {
				cmbCategoria.setValue(pacoteTarifa.getCategoria().getDesccategoria());
			}
		}
	}

	public void onClick$btnSalvar(Event ev) throws InterruptedException {
		chkIndvendaweb.getValue();
		
		try {
			
			tipoTarifaPacote.setDesctipotarifa(txtDescTarifa.getValue());

			Categoria categoria = null;
			if(cmbCategoria.getSelectedItem() != null) {
				categoria = (Categoria) cmbCategoria.getSelectedItem().getValue();
			}

			pacoteTarifa = pacoteTarifaService.vincularPacoteTarifa(pacoteTarifa, pacote, tipoTarifaPacote,
					tarifa.getValueDecimal(), chkIndvendaweb.isChecked(), categoria);
			
			tarifaPacoteList.updateItem(pacoteTarifa);

			closeWindow();

		} catch (WrongValueException ex) {
			throw (WrongValueException) ex;
		} catch (Exception ex) {
			log.error(ex);
			Messagebox.show(Labels.getLabel("MSG.Error").concat(" - ").concat(ex.toString()),
					Labels.getLabel("editarConfiguracionPacoteController.window.title"),
					Messagebox.OK, Messagebox.ERROR);
		}
	}

	public List<Categoria> getLsCategorias() {
		return lsCategorias;
	}

	public void setLsCategorias(List<Categoria> lsCategorias) {
		this.lsCategorias = lsCategorias;
	}
}
